exports.loadDefaultTypeConverters = function(context){
	var stringConverter = function(value, inputType) { return new String(value).valueOf() };
	context.registerTypeConversion("xsd:string", stringConverter);
	var booleanConverter = function(value, inputType) { switch(value){case "false":case "0":return false;} return(new Boolean(value)).valueOf() };
	context.registerTypeConversion("xsd:boolean", booleanConverter);
	var numberConverter = function(value, inputType) { return(new Number(value)).valueOf() };
	var floatConverter = function(value, inputType) {
		switch(value){
			case "INF": return Number.POSITIVE_INFINITY;
			case "-INF": return Number.NEGATIVE_INFINITY;
			default: return numberConverter(value, inputType);
		};
	};
	context.registerTypeConversion("xsd:float", floatConverter);
	context.registerTypeConversion("xsd:integer", numberConverter);
	context.registerTypeConversion("xsd:long", numberConverter);
	context.registerTypeConversion("xsd:double", numberConverter);
	context.registerTypeConversion("xsd:decimal", numberConverter);
	context.registerTypeConversion("xsd:nonPositiveInteger", numberConverter);
	context.registerTypeConversion("xsd:nonNegativeInteger", numberConverter);
	context.registerTypeConversion("xsd:negativeInteger", numberConverter);
	context.registerTypeConversion("xsd:int", numberConverter);
	context.registerTypeConversion("xsd:unsignedLong", numberConverter);
	context.registerTypeConversion("xsd:positiveInteger", numberConverter);
	context.registerTypeConversion("xsd:short", numberConverter);
	context.registerTypeConversion("xsd:unsignedInt", numberConverter);
	context.registerTypeConversion("xsd:byte", numberConverter);
	context.registerTypeConversion("xsd:unsignedShort", numberConverter);
	context.registerTypeConversion("xsd:unsignedByte", numberConverter);
	var dateConverter = function(value, inputType) { return new Date(value) };
	context.registerTypeConversion("xsd:date", dateConverter);
	context.registerTypeConversion("xsd:time", dateConverter);
	context.registerTypeConversion("xsd:dateTime", dateConverter);
};
exports.loadRequiredPrefixMap = function(context){
	context.setPrefix("owl", "http://www.w3.org/2002/07/owl#");
	context.setPrefix("rdf", "http://www.w3.org/1999/02/22-rdf-syntax-ns#");
	context.setPrefix("rdfs", "http://www.w3.org/2000/01/rdf-schema#");
	context.setPrefix("rdfa", "http://www.w3.org/ns/rdfa#");
	context.setPrefix("xhv", "http://www.w3.org/1999/xhtml/vocab#");
	context.setPrefix("xml", "http://www.w3.org/XML/1998/namespace");
	context.setPrefix("xsd", "http://www.w3.org/2001/XMLSchema#");
};
exports.loadDefaultPrefixMap = function(context){
	exports.loadRequiredPrefixMap(context);
	context.setPrefix("grddl", "http://www.w3.org/2003/g/data-view#");
	context.setPrefix("powder", "http://www.w3.org/2007/05/powder#");
	context.setPrefix("powders", "http://www.w3.org/2007/05/powder-s#");
	context.setPrefix("rif", "http://www.w3.org/2007/rif#");
	context.setPrefix("atom", "http://www.w3.org/2005/Atom/");
	context.setPrefix("xhtml", "http://www.w3.org/1999/xhtml#");
	context.setPrefix("formats", "http://www.w3.org/ns/formats/");
	context.setPrefix("xforms", "http://www.w3.org/2002/xforms/");
	context.setPrefix("xhtmlvocab", "http://www.w3.org/1999/xhtml/vocab/");
	context.setPrefix("xpathfn", "http://www.w3.org/2005/xpath-functions#");
	context.setPrefix("http", "http://www.w3.org/2006/http#");
	context.setPrefix("link", "http://www.w3.org/2006/link#");
	context.setPrefix("time", "http://www.w3.org/2006/time#");
	context.setPrefix("acl", "http://www.w3.org/ns/auth/acl#");
	context.setPrefix("cert", "http://www.w3.org/ns/auth/cert#");
	context.setPrefix("rsa", "http://www.w3.org/ns/auth/rsa#");
	context.setPrefix("crypto", "http://www.w3.org/2000/10/swap/crypto#");
	context.setPrefix("list", "http://www.w3.org/2000/10/swap/list#");
	context.setPrefix("log", "http://www.w3.org/2000/10/swap/log#");
	context.setPrefix("math", "http://www.w3.org/2000/10/swap/math#");
	context.setPrefix("os", "http://www.w3.org/2000/10/swap/os#");
	context.setPrefix("string", "http://www.w3.org/2000/10/swap/string#");
	context.setPrefix("doc", "http://www.w3.org/2000/10/swap/pim/doc#");
	context.setPrefix("contact", "http://www.w3.org/2000/10/swap/pim/contact#");
	context.setPrefix("p3p", "http://www.w3.org/2002/01/p3prdfv1#");
	context.setPrefix("swrl", "http://www.w3.org/2003/11/swrl#");
	context.setPrefix("swrlb", "http://www.w3.org/2003/11/swrlb#");
	context.setPrefix("exif", "http://www.w3.org/2003/12/exif/ns#");
	context.setPrefix("earl", "http://www.w3.org/ns/earl#");
	context.setPrefix("ma", "http://www.w3.org/ns/ma-ont#");
	context.setPrefix("sawsdl", "http://www.w3.org/ns/sawsdl#");
	context.setPrefix("sd", "http://www.w3.org/ns/sparql-service-description#");
	context.setPrefix("skos", "http://www.w3.org/2004/02/skos/core#");
	context.setPrefix("fresnel", "http://www.w3.org/2004/09/fresnel#");
	context.setPrefix("gen", "http://www.w3.org/2006/gen/ont#");
	context.setPrefix("timezone", "http://www.w3.org/2006/timezone#");
	context.setPrefix("skosxl", "http://www.w3.org/2008/05/skos-xl#");
	context.setPrefix("org", "http://www.w3.org/ns/org#");
	context.setPrefix("ical", "http://www.w3.org/2002/12/cal/ical#");
	context.setPrefix("wgs84", "http://www.w3.org/2003/01/geo/wgs84_pos#");
	context.setPrefix("vcard", "http://www.w3.org/2006/vcard/ns#");
	context.setPrefix("turtle", "http://www.w3.org/2008/turtle#");
	context.setPrefix("pointers", "http://www.w3.org/2009/pointers#");
	context.setPrefix("dcat", "http://www.w3.org/ns/dcat#");
	context.setPrefix("imreg", "http://www.w3.org/2004/02/image-regions#");
	context.setPrefix("rdfg", "http://www.w3.org/2004/03/trix/rdfg-1/");
	context.setPrefix("swp", "http://www.w3.org/2004/03/trix/swp-2/");
	context.setPrefix("rei", "http://www.w3.org/2004/06/rei#");
	context.setPrefix("wairole", "http://www.w3.org/2005/01/wai-rdf/GUIRoleTaxonomy#");
	context.setPrefix("states", "http://www.w3.org/2005/07/aaa#");
	context.setPrefix("wn20schema", "http://www.w3.org/2006/03/wn/wn20/schema/");
	context.setPrefix("httph", "http://www.w3.org/2007/ont/httph#");
	context.setPrefix("act", "http://www.w3.org/2007/rif-builtin-action#");
	context.setPrefix("common", "http://www.w3.org/2007/uwa/context/common.owl#");
	context.setPrefix("dcn", "http://www.w3.org/2007/uwa/context/deliverycontext.owl#");
	context.setPrefix("hard", "http://www.w3.org/2007/uwa/context/hardware.owl#");
	context.setPrefix("java", "http://www.w3.org/2007/uwa/context/java.owl#");
	context.setPrefix("loc", "http://www.w3.org/2007/uwa/context/location.owl#");
	context.setPrefix("net", "http://www.w3.org/2007/uwa/context/network.owl#");
	context.setPrefix("push", "http://www.w3.org/2007/uwa/context/push.owl#");
	context.setPrefix("soft", "http://www.w3.org/2007/uwa/context/software.owl#");
	context.setPrefix("web", "http://www.w3.org/2007/uwa/context/web.owl#");
	context.setPrefix("content", "http://www.w3.org/2008/content#");
	context.setPrefix("vs", "http://www.w3.org/2003/06/sw-vocab-status/ns#");
	context.setPrefix("air", "http://dig.csail.mit.edu/TAMI/2007/amord/air#");
	context.setPrefix("ex", "http://example.org/");
	context.setPrefix("dc", "http://purl.org/dc/terms/");
	context.setPrefix("dc11", "http://purl.org/dc/elements/1.1/");
	context.setPrefix("dctype", "http://purl.org/dc/dcmitype/");
	context.setPrefix("foaf", "http://xmlns.com/foaf/0.1/");
	context.setPrefix("cc", "http://creativecommons.org/ns#");
	context.setPrefix("opensearch", "http://a9.com/-/spec/opensearch/1.1/");
	context.setPrefix("void", "http://rdfs.org/ns/void#");
	context.setPrefix("sioc", "http://rdfs.org/sioc/ns#");
	context.setPrefix("sioca", "http://rdfs.org/sioc/actions#");
	context.setPrefix("sioct", "http://rdfs.org/sioc/types#");
	context.setPrefix("lgd", "http://linkedgeodata.org/vocabulary#");
	context.setPrefix("moat", "http://moat-project.org/ns#");
	context.setPrefix("days", "http://ontologi.es/days#");
	context.setPrefix("giving", "http://ontologi.es/giving#");
	context.setPrefix("lang", "http://ontologi.es/lang/core#");
	context.setPrefix("like", "http://ontologi.es/like#");
	context.setPrefix("status", "http://ontologi.es/status#");
	context.setPrefix("og", "http://opengraphprotocol.org/schema/");
	context.setPrefix("protege", "http://protege.stanford.edu/system#");
	context.setPrefix("dady", "http://purl.org/NET/dady#");
	context.setPrefix("uri", "http://purl.org/NET/uri#");
	context.setPrefix("audio", "http://purl.org/media/audio#");
	context.setPrefix("video", "http://purl.org/media/video#");
	context.setPrefix("gridworks", "http://purl.org/net/opmv/types/gridworks#");
	context.setPrefix("hcterms", "http://purl.org/uF/hCard/terms/");
	context.setPrefix("bio", "http://purl.org/vocab/bio/0.1/");
	context.setPrefix("cs", "http://purl.org/vocab/changeset/schema#");
	context.setPrefix("geographis", "http://telegraphis.net/ontology/geography/geography#");
	context.setPrefix("doap", "http://usefulinc.com/ns/doap#");
	context.setPrefix("daml", "http://www.daml.org/2001/03/daml+oil#");
	context.setPrefix("geonames", "http://www.geonames.org/ontology#");
	context.setPrefix("sesame", "http://www.openrdf.org/schema/sesame#");
	context.setPrefix("cv", "http://rdfs.org/resume-rdf/");
	context.setPrefix("wot", "http://xmlns.com/wot/0.1/");
	context.setPrefix("media", "http://purl.org/microformat/hmedia/");
	context.setPrefix("ctag", "http://commontag.org/ns#");
};
